:root {
    --font-family: system-ui, -apple-system, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    --body-color: #333;
    --body-background: #fff;
    --hr-border-color: #ccc;
    --title-color: #333;
    --link-color: #3366CC;
    --link-focus-color: red;
    --link-hover-color: #333;
    --link-visited-color: purple;

    --header-list-border-color: #ddd;
    --header-link-color: #444;
    --header-link-focus-color: #888;
    --header-link-hover-color: #888;
    --header-active-link-color: #444;

    --page-header-title-border-color: #333;

    --logo-color: #000;
    --logo-hover-color-span: #000;

    --table-border-color: #ddd;
    --table-th-background: #fcfcfc;
    --table-th-color: #333;
    --table-tr-hover-background-color: #f9f9f9;
    --table-tr-hover-color: #333;

    --button-primary-border-color: #3079ed;
    --button-primary-background: #4d90fe;
    --button-primary-color: #fff;
    --button-primary-focus-border-color: #2f5bb7;
    --button-primary-focus-background: #357ae8;

    --input-border: 1px solid #ccc;
    --input-background: #fff;
    --input-color: #333;
    --input-placeholder-color: #d0d0d0;

    --input-focus-color: #000;
    --input-focus-border-color: rgba(82, 168, 236, 0.8);
    --input-focus-box-shadow: 0 0 8px rgba(82, 168, 236, 0.6);

    --alert-color: #c09853;
    --alert-background-color: #fcf8e3;
    --alert-border-color: #fbeed5;

    --alert-success-color: #468847;
    --alert-success-background-color: #dff0d8;
    --alert-success-border-color: #d6e9c6;

    --alert-error-color: #b94a48;
    --alert-error-background-color: #f2dede;
    --alert-error-border-color: #eed3d7;

    --alert-info-color: #3a87ad;
    --alert-info-background-color: #d9edf7;
    --alert-info-border-color: #bce8f1;

    --panel-background: #fcfcfc;
    --panel-border-color: #ddd;
    --panel-color: #333;

    --pagination-link-color: #333;
    --pagination-border-color: #ddd;

    --category-color: #333;
    --category-background-color: #fffcd7;
    --category-border-color: #d5d458;
    --category-link-color: #000;
    --category-link-hover-color: #000;

    --item-border-color: #ddd;
    --item-padding: 5px;
    --item-title-link-font-weight: 600;

    --item-status-read-title-link-color: #777;
    --item-status-read-title-focus-color: #777;

    --item-meta-focus-color: #777;
    --item-meta-li-color: #aaa;

    --current-item-border-width: 3px;
    --current-item-border-color: #bce;
    --current-item-box-shadow: none;

    --entry-header-border-color: #ddd;
    --entry-header-title-link-color: #333;
    --entry-content-color: #555;
    --entry-content-code-color: #333;
    --entry-content-code-background: #f0f0f0;
    --entry-content-code-border-color: #ddd;
    --entry-content-quote-color: #666;
    --entry-content-abbr-border-color: #999;
    --entry-content-aside-border-color: #D3D3D3;
    --entry-enclosure-border-color: #333;

    --parsing-error-color: #333;
    --feed-parsing-error-background-color: #fcf8e3;
    --feed-parsing-error-border-style: solid;
    --feed-parsing-error-border-color: #f9e883;

    --feed-has-unread-background-color: #dfd;
    --feed-has-unread-border-style: solid;
    --feed-has-unread-border-color: #bee6bc;

    --category-has-unread-background-color: #dfd;
    --category-has-unread-border-style: solid;
    --category-has-unread-border-color: #bee6bc;

    --keyboard-shortcuts-li-color: #333;

    --counter-color: #666;
}

html {
    color-scheme: light;
}

@media (prefers-color-scheme: dark) {
    :root {
        --body-color: #efefef;
        --body-background: #222;
        --hr-border-color: #555;
        --title-color: #aaa;
        --link-color: #aaa;
        --link-focus-color: #ddd;
        --link-hover-color: #ddd;
        --link-visited-color: #f083e4;

        --header-list-border-color: #333;
        --header-link-color: #ddd;
        --header-link-focus-color: rgba(82, 168, 236, 0.85);
        --header-link-hover-color: rgba(82, 168, 236, 0.85);
        --header-active-link-color: #9b9494;

        --page-header-title-border-color: #333;

        --logo-color: #bbb;
        --logo-hover-color-span: #bbb;

        --table-border-color: #555;
        --table-th-background: #333;
        --table-th-color: #aaa;
        --table-tr-hover-background-color: #333;
        --table-tr-hover-color: #aaa;

        --button-primary-border-color: #444;
        --button-primary-background: #333;
        --button-primary-color: #efefef;
        --button-primary-focus-border-color: #888;
        --button-primary-focus-background: #555;

        --input-border: 1px solid #555;
        --input-background: #333;
        --input-color: #ccc;
        --input-placeholder-color: #666;

        --input-focus-color: #efefef;
        --input-focus-border-color: rgba(82, 168, 236, 0.8);
        --input-focus-box-shadow: 0 0 8px rgba(82, 168, 236, 0.6);

        --alert-color: #efefef;
        --alert-background-color: #333;
        --alert-border-color: #444;

        --alert-success-color: #efefef;
        --alert-success-background-color: #333;
        --alert-success-border-color: #444;

        --alert-error-color: #efefef;
        --alert-error-background-color: #333;
        --alert-error-border-color: #444;

        --alert-info-color: #efefef;
        --alert-info-background-color: #333;
        --alert-info-border-color: #444;

        --panel-background: #333;
        --panel-border-color: #555;
        --panel-color: #9b9b9b;

        --modal-background: #333;
        --modal-color: #efefef;
        --modal-box-shadow: 0 0 10px rgba(82, 168, 236, 0.6);

        --pagination-link-color: #aaa;
        --pagination-border-color: #333;

        --category-color: #efefef;
        --category-background-color: #333;
        --category-border-color: #444;
        --category-link-color: #999;
        --category-link-hover-color: #aaa;

        --item-border-color: #666;
        --item-padding: 4px;
        --item-title-link-font-weight: 400;

        --item-status-read-title-link-color: #666;
        --item-status-read-title-focus-color: rgba(82, 168, 236, 0.6);

        --item-meta-focus-color: #aaa;
        --item-meta-li-color: #ddd;

        --current-item-border-width: 2px;
        --current-item-border-color: rgba(82, 168, 236, 0.8);
        --current-item-box-shadow: 0 0 8px rgba(82, 168, 236, 0.6);

        --entry-header-border-color: #333;
        --entry-header-title-link-color: #bbb;
        --entry-content-color: #999;
        --entry-content-code-color: #fff;
        --entry-content-code-background: #555;
        --entry-content-code-border-color: #888;
        --entry-content-quote-color: #777;
        --entry-content-abbr-border-color: #777;
        --entry-content-aside-border-color: #777;
        --entry-enclosure-border-color: #333;

        --parsing-error-color: #eee;
        --feed-parsing-error-background-color: #3a1515;
        --feed-parsing-error-border-style: solid;
        --feed-parsing-error-border-color: #562222;

        --feed-has-unread-background-color: #1b1a1a;
        --feed-has-unread-border-style: solid;
        --feed-has-unread-border-color: rgb(33 57 76);

        --category-has-unread-background-color: #1b1a1a;
        --category-has-unread-border-style: solid;
        --category-has-unread-border-color: rgb(33 57 76);

        --keyboard-shortcuts-li-color: #9b9b9b;

        --counter-color: #bbb;
    }

    html {
        color-scheme: dark;
    }
}
